@@ -1,4 +1,4 @@
-# Copyright (c) 2018-2019 ISciences, LLC.
+# Copyright (c) 2018-2020 ISciences, LLC.
 # All rights reserved.
 #
 # This software is licensed under the Apache License, Version 2.0 (the "License").
@@ -22,9 +22,8 @@ if (!isGeneric('coverage_fraction')) {
   } else if(is.na(sf::st_crs(y)) && !is.na(sf::st_crs(x))) {
     warning("No CRS specified for polygons; assuming they have the same CRS as the raster.")
   } else if(sf::st_crs(x) != sf::st_crs(y)) {
-    old_crs <- sf::st_crs(y)
     y <- sf::st_transform(y, sf::st_crs(x))
-    warning("Polygons transformed from EPSG:", old_crs$epsg, " to EPSG:", sf::st_crs(x)$epsg)
+    warning("Polygons transformed to raster CRS (EPSG:", sf::st_crs(x)$epsg, ")")
   }
 
   lapply(sf::st_as_binary(y), function(wkb) {
